from fileformats.extras.testing import EncodedToTextConverter
from fileformats.testing import EncodedText
from fileformats.text import TextFile
from pydra.compose import python

from frametree.core.pipeline import RuntimeConverterWorkflow
from frametree.file_system import FileSystem
from frametree.testing import TestAxes
from frametree.testing.blueprint import FileSetEntryBlueprint as FileBP
from frametree.testing.blueprint import TestDatasetBlueprint


@python.define(outputs=["out_file"])
def EncodedTextIdentity(in_file: EncodedText) -> EncodedText:
    assert in_file.raw_contents != "file.txt"
    return in_file


def test_pipeline_union_column_datatype(saved_dataset, data_store, work_dir):

    bp = TestDatasetBlueprint(
        hierarchy=[
            "abcd"
        ],  # e.g. XNAT where session ID is unique in project but final layer is organised by visit
        axes=TestAxes,
        dim_lengths=[1, 1, 1, 1],
        entries=[
            FileBP(path="file", datatype=TextFile, filenames=["file.txt"]),
        ],
    )
    frameset = bp.make_dataset(FileSystem(), str(work_dir / "dataset"))
    frameset.add_source(
        "file",
        EncodedText.convertible_from(),  # Union datatype
    )
    frameset.add_sink(
        "out",
        TextFile,
    )

    # Start generating the arguments for the CLI
    # Add source to loaded dataset

    frameset.apply(
        "a_pipeline",
        EncodedTextIdentity(),
        inputs={
            (
                "file",
                "in_file",
                EncodedText,
            )
        },
        outputs=[
            (
                "out",
                "out_file",
                EncodedText,
            )
        ],
    )

    pipeline = frameset.pipelines["a_pipeline"]
    wf = pipeline()

    per_row = wf.construct()["PipelineRowWorkflow"]._task.construct()
    input_converter = per_row["file_input_converter"]
    output_converter = per_row["out_output_converter"]
    assert isinstance(input_converter._task, RuntimeConverterWorkflow)
    assert isinstance(output_converter._task, EncodedToTextConverter)

    out = next(iter(frameset.derive("out")[0]))
    assert out.raw_contents == "file.txt"



from frametree.core.axes import Axes as BaseAxes


class Samples(BaseAxes):
    """
    The most basic data space within only one dimension
    """

    # Root row of the dataset
    constant = 0b0  # constant across the dataset

    # Axes of the data space
    sample = 0b1


# Set default axes for namespace
Axes = Samples



from frametree.axes.medimage import MedImage


def test_is_parent():
    assert not MedImage.session.is_parent(MedImage.session)
    assert not MedImage.session.is_parent(MedImage.subject)
    assert not MedImage.session.is_parent(MedImage.member)
    assert not MedImage.session.is_parent(MedImage.group)
    assert not MedImage.session.is_parent(MedImage.visit)
    assert not MedImage.session.is_parent(MedImage.groupedvisit)
    assert not MedImage.session.is_parent(MedImage.matchedvisit)
    assert not MedImage.session.is_parent(MedImage.constant)

    assert MedImage.member.is_parent(MedImage.session)
    assert MedImage.member.is_parent(MedImage.subject)
    assert not MedImage.member.is_parent(MedImage.member)
    assert not MedImage.member.is_parent(MedImage.group)
    assert not MedImage.member.is_parent(MedImage.visit)
    assert not MedImage.member.is_parent(MedImage.groupedvisit)
    assert MedImage.member.is_parent(MedImage.matchedvisit)
    assert not MedImage.member.is_parent(MedImage.constant)

    assert MedImage.group.is_parent(MedImage.session)
    assert MedImage.group.is_parent(MedImage.subject)
    assert not MedImage.group.is_parent(MedImage.member)
    assert not MedImage.group.is_parent(MedImage.group)
    assert not MedImage.group.is_parent(MedImage.visit)
    assert MedImage.group.is_parent(MedImage.groupedvisit)
    assert not MedImage.group.is_parent(MedImage.matchedvisit)
    assert not MedImage.group.is_parent(MedImage.constant)

    assert MedImage.visit.is_parent(MedImage.session)
    assert not MedImage.visit.is_parent(MedImage.subject)
    assert not MedImage.visit.is_parent(MedImage.member)
    assert not MedImage.visit.is_parent(MedImage.group)
    assert not MedImage.visit.is_parent(MedImage.visit)
    assert MedImage.visit.is_parent(MedImage.groupedvisit)
    assert MedImage.visit.is_parent(MedImage.matchedvisit)
    assert not MedImage.visit.is_parent(MedImage.constant)

    assert MedImage.subject.is_parent(MedImage.session)
    assert not MedImage.subject.is_parent(MedImage.subject)
    assert not MedImage.subject.is_parent(MedImage.member)
    assert not MedImage.subject.is_parent(MedImage.group)
    assert not MedImage.subject.is_parent(MedImage.visit)
    assert not MedImage.subject.is_parent(MedImage.groupedvisit)
    assert not MedImage.subject.is_parent(MedImage.matchedvisit)
    assert not MedImage.subject.is_parent(MedImage.constant)

    assert MedImage.groupedvisit.is_parent(MedImage.session)
    assert not MedImage.groupedvisit.is_parent(MedImage.subject)
    assert not MedImage.groupedvisit.is_parent(MedImage.member)
    assert not MedImage.groupedvisit.is_parent(MedImage.group)
    assert not MedImage.groupedvisit.is_parent(MedImage.visit)
    assert not MedImage.groupedvisit.is_parent(MedImage.groupedvisit)
    assert not MedImage.groupedvisit.is_parent(MedImage.matchedvisit)
    assert not MedImage.groupedvisit.is_parent(MedImage.constant)

    assert MedImage.matchedvisit.is_parent(MedImage.session)
    assert not MedImage.matchedvisit.is_parent(MedImage.subject)
    assert not MedImage.matchedvisit.is_parent(MedImage.member)
    assert not MedImage.matchedvisit.is_parent(MedImage.group)
    assert not MedImage.matchedvisit.is_parent(MedImage.visit)
    assert not MedImage.matchedvisit.is_parent(MedImage.groupedvisit)
    assert not MedImage.matchedvisit.is_parent(MedImage.matchedvisit)
    assert not MedImage.matchedvisit.is_parent(MedImage.constant)

    assert MedImage.constant.is_parent(MedImage.session)
    assert MedImage.constant.is_parent(MedImage.subject)
    assert MedImage.constant.is_parent(MedImage.member)
    assert MedImage.constant.is_parent(MedImage.group)
    assert MedImage.constant.is_parent(MedImage.visit)
    assert MedImage.constant.is_parent(MedImage.groupedvisit)
    assert MedImage.constant.is_parent(MedImage.matchedvisit)
    assert not MedImage.constant.is_parent(MedImage.constant)
